package com.jiayue.insu.incloud.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.jiayue.insu.common.core.util.R;
import com.jiayue.insu.incloud.entity.IntegrationCompany;
import com.jiayue.insu.incloud.entity.Quartz;
import com.jiayue.insu.incloud.service.IntegrationCompanyService;
import com.jiayue.insu.incloud.service.QuartzService;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.*;


/**
 * QuartzController
 *
 * @author yh
 * @date 2022-03-18 15:48:48
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/quartz")
public class QuartzController {

    private final QuartzService quartzService;

    /**
     * 分页查询
     *
     * @param page   分页对象
     * @param quartz quartz
     * @return
     */

    @GetMapping("/page")
    public R getStationPage(Page page, Quartz quartz) {
        return R.ok(quartzService.page(page, Wrappers.query(quartz)));
    }

    @GetMapping("/all")
    public R getAll() {
        return R.ok(quartzService.list());
    }

    /**
     * 通过code查询定时任务
     *
     * @param jobName jobName
     * @return R
     */
    @GetMapping("/{jobName}")
    public R getByCode(@PathVariable("jobName") String  jobName) {
        return R.ok(quartzService.findByJobName(jobName));
    }


    /**
     * 新增quartz
     *
     * @param quartz quartz
     * @return R
     */

    @PostMapping
    public R save(@RequestBody  Quartz quartz) {
        return R.ok(quartzService.save(quartz));
    }

    /**
     * 修改quartz
     *
     * @param quartz quartz
     * @return R
     */

    @PutMapping
    public R updateById(@RequestBody Quartz quartz) {
        return R.ok(quartzService.updateById(quartz));
    }

    /**
     * 通过id删除
     *
     * @param id id
     * @return R
     */
    @DeleteMapping("/{id}")
    public R removeById(@PathVariable Integer id) {
        return R.ok(quartzService.removeById(id));
    }


}
